@@ -31,7 +31,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.96';
+$VERSION = '2.00';
 
 sub ProcessFujiDir($$$);
 sub ProcessFaceRec($$$);
@@ -426,6 +426,16 @@ my %faceCategories = (
             0x300 => 'DR (Dynamic Range priority)',
         },
     },
+    0x1037 => { #forum17591
+        Name => 'MultipleExposure',
+        Writable => 'int16u', # (NC)
+        PrintConv => {
+            1 => 'Additive',
+            2 => 'Average',
+            3 => 'Light',
+            4 => 'Dark',
+        },
+    },
     0x1040 => { #8
         Name => 'ShadowTone',
         Writable => 'int32s',
@@ -586,16 +596,44 @@ my %faceCategories = (
         Name => 'SequenceNumber',
         Writable => 'int16u',
     },
+    0x1102 => { #forum17602
+        Name => 'WhiteBalanceBracketing',
+        Writable => 'int16u', # (NC)
+        PrintHex => 1,
+        PrintConv => {
+            0x01ff => '+/- 1',
+            0x02ff => '+/- 2',
+            0x03ff => '+/- 3',
+        },
+    },
     0x1103 => {
         Name => 'DriveSettings',
         SubDirectory => { TagTable => 'Image::ExifTool::FujiFilm::DriveSettings' },
     },
     0x1105 => { Name => 'PixelShiftShots',  Writable => 'int16u' }, #IB
     0x1106 => { Name => 'PixelShiftOffset', Writable => 'rational64s', Count => 2 }, #IB
-    # (0x1150-0x1152 exist only for Pro Low-light and Pro Focus PictureModes)
-    # 0x1150 - Pro Low-light - val=1; Pro Focus - val=2 (ref 7); HDR - val=128 (forum10799)
-    # 0x1151 - Pro Low-light - val=4 (number of pictures taken?); Pro Focus - val=2,3 (ref 7); HDR - val=3 (forum10799)
-    # 0x1152 - Pro Low-light - val=1,3,4 (stacked pictures used?); Pro Focus - val=1,2 (ref 7); HDR - val=3 (forum10799)
+    0x1150 => {
+        Name => 'CompositeImageMode',
+        Writable => 'int32u',
+        PrintConv => {
+            0 => 'n/a', #PH
+            1 => 'Pro Low-light', #7
+            2 => 'Pro Focus', #7
+            32 => 'Panorama', #PH
+            128 => 'HDR', #forum10799
+            1024 => 'Multi-exposure', #forum17591
+        },
+    },
+    0x1151 => {
+        Name => 'CompositeImageCount1',
+        Writable => 'int16u',
+        # Pro Low-light - val=4 (number of pictures taken?); Pro Focus - val=2,3 (ref 7); HDR - val=3 (forum10799)
+    },
+    0x1152 => {
+        Name => 'CompositeImageCount2',
+        Writable => 'int16u',
+        # Pro Low-light - val=1,3,4 (stacked pictures used?); Pro Focus - val=1,2 (ref 7); HDR - val=3 (forum10799)
+    },
     0x1153 => { #forum7668
         Name => 'PanoramaAngle',
         Writable => 'int16u',
@@ -605,8 +643,8 @@ my %faceCategories = (
         Writable => 'int16u',
         PrintConv => {
             1 => 'Right',
-            2 => 'Up',
-            3 => 'Left',
+            2 => 'Left', #forum17591
+            3 => 'Up', #forum17591
             4 => 'Down',
         },
     },
@@ -628,6 +666,10 @@ my %faceCategories = (
             0x60006 => 'Partial Color Purple',
             0x70000 => 'Soft Focus',
             0x90000 => 'Low Key',
+            0x100000 => 'Light Leak', #forum17392
+            0x130000 => 'Expired Film Green', #forum17392
+            0x130001 => 'Expired Film Red', #forum17392 (NC)
+            0x130002 => 'Expired Film Neutral', #forum17392
         },
     },
     0x1210 => { #2
@@ -828,6 +870,13 @@ my %faceCategories = (
     },
     0x1447 => { Name => 'FujiModel',  Writable => 'string' },
     0x1448 => { Name => 'FujiModel2', Writable => 'string' },
+
+    # Found in X-M5, X-E5
+    # White balance as shot. Same valus as 0xf00e.
+    0x144a => { Name => 'WBRed',      Writable => 'int16u' },
+    0x144b => { Name => 'WBGreen',    Writable => 'int16u' },
+    0x144c => { Name => 'WBBlue',     Writable => 'int16u' },
+    
     0x144d => { Name => 'RollAngle',  Writable => 'rational64s' }, #forum14319
     0x3803 => { #forum10037
         Name => 'VideoRecordingMode',
@@ -911,6 +960,7 @@ my %faceCategories = (
             3 => 'Right Eye',
             7 => 'Body',
             8 => 'Head',
+            9 => 'Both Eyes', #forum17635
             11 => 'Bike',
             12 => 'Body of Car',
             13 => 'Front of Car',
@@ -1141,7 +1191,7 @@ my %faceCategories = (
 %Image::ExifTool::FujiFilm::FaceRecInfo = (
     PROCESS_PROC => \&ProcessFaceRec,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
-    VARS => { NO_ID => 1 },
+    VARS => { ID_FMT => 'none' },
     NOTES => 'Face recognition information.',
     Face1Name => { },
     Face2Name => { },
@@ -1547,20 +1597,21 @@ my %faceCategories = (
     TAG_PREFIX => 'MRAW',
     NOTES => q{
         Tags extracted from the M-RAW header of multi-image RAF files.  The family 1
-        group name for these tags is "M-RAW".
+        group name for these tags is "M-RAW".  Additional metadata may be extracted
+        from the embedded RAW images with the ExtractEmbedded option.
     },
-    1 => { Name => 'RawImageNumber', Format => 'int32u' },
-    # 3 - seen "0 100", "-300 100" and "300 100" for a sequence of 3 images
-    3 => { Name => 'ExposureCompensation', Format => 'rational32s', Unknown => 1, Hidden => 1, PrintConv => 'sprintf("%+.2f",$val)' },
-    # 4 - (same value as 3 in all my samples)
-    4 => { Name => 'ExposureCompensation2', Format => 'rational32s', Unknown => 1, Hidden => 1, PrintConv => 'sprintf("%+.2f",$val)' },
-    # 5 - seen "10 1600", "10 6800", "10 200", "10 35000" etc
-    5 => { Name => 'ExposureTime', Format => 'rational64u', PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)' },
-    # 6 - seen "450 100", "400 100" (all images in RAF have same value)
-    6 => { Name => 'FNumber', Format => 'rational64u', PrintConv => 'Image::ExifTool::Exif::PrintFNumber($val)' },
-    # 7 - seen 200, 125, 250, 2000
-    7 => 'ISO',
-    # 8 - seen 0
+    0x2001 => { Name => 'RawImageNumber', Format => 'int32u' },
+    # 0x2003 - seen "0 100", "-300 100" and "300 100" for a sequence of 3 images
+    0x2003 => { Name => 'ExposureCompensation', Format => 'rational32s', Unknown => 1, Hidden => 1, PrintConv => 'sprintf("%+.2f",$val)' },
+    # 0x2004 - (same value as 3 in all my samples)
+    0x2004 => { Name => 'ExposureCompensation2', Format => 'rational32s', Unknown => 1, Hidden => 1, PrintConv => 'sprintf("%+.2f",$val)' },
+    # 0x2005 - seen "10 1600", "10 6800", "10 200", "10 35000" etc
+    0x2005 => { Name => 'ExposureTime', Format => 'rational64u', PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)' },
+    # 0x2006 - seen "450 100", "400 100" (all images in RAF have same value)
+    0x2006 => { Name => 'FNumber', Format => 'rational64u', PrintConv => 'Image::ExifTool::Exif::PrintFNumber($val)' },
+    # 0x2007 - seen 200, 125, 250, 2000
+    0x2007 => 'ISO',
+    # 0x2008 - seen 0, 65536
 );
 
 #------------------------------------------------------------------------------
@@ -1669,10 +1720,11 @@ sub ProcessFujiDir($$$)
 #------------------------------------------------------------------------------
 # get information from FujiFilm M-RAW header
 # Inputs: 0) ExifTool ref, 1) dirInfo ref, 2) tag table ref
-# Returns: 1 if this was a valid M-RAW headerx
+# Returns: 1 if this was a valid M-RAW header
 sub ProcessMRAW($$$)
 {
     my ($et, $dirInfo, $tagTablePtr) = @_;
+    return 1 if $$et{DOC_NUM};
     my $dataPt = $$dirInfo{DataPt};
     my $dataPos = $$dirInfo{DataPos};
     my $dataLen = length $$dataPt;
@@ -1686,15 +1738,16 @@ sub ProcessMRAW($$$)
     my $pos = 44;
     my ($i, $n);
     for ($n=0; ; ++$n) {
-        $pos += 16;  # skip offset/size fields
-        my $end = $pos + $size;
+        my $end = $pos + 16 + $size;
         last if $end > $dataLen;
+        my $rafStart = Get64u($dataPt, $pos);
+        my $rafLen = Get64u($dataPt, $pos+8);
+        $pos += 16;  # skip offset/size fields
         $$et{DOC_NUM} = ++$$et{DOC_COUNT} if $pos > 60;
         $et->VPrint(0, "$$et{INDENT}(Raw image $n parameters: $size bytes, $num entries)\n");
         for ($i=0; $i<$num; ++$i) {
             last if $pos + 4 > $end;
-            # (don't know what the byte at the current $pos is for, value = 0x20)
-            my $tag = Get8u($dataPt, $pos+1);
+            my $tag = Get16u($dataPt, $pos);
             my $size = Get16u($dataPt, $pos+2);
             $pos += 4;
             last if $pos + $size > $end;
@@ -1706,6 +1759,21 @@ sub ProcessMRAW($$$)
             );
             $pos += $size;
         }
+        if ($rafStart and $et->Options('ExtractEmbedded')) {
+            if ($et->Options('Verbose')) {
+                my $msg = sprintf("$$et{INDENT}(RAW image $n data: Start=0x%x, Length=0x%x)\n",$rafStart,$rafLen);
+                $et->VPrint(0, $msg);
+            }
+            my $raf = $$et{RAF};
+            my $tell = $raf->Tell();
+            my $order = GetByteOrder();
+            my $fujiWidth = $$et{FujiWidth};
+            $raf->Seek($rafStart, 0) or next;
+            ProcessRAF($et, { RAF => $raf, Base => $rafStart });
+            $$et{FujiWidth} = $fujiWidth;
+            SetByteOrder($order);
+            $raf->Seek($tell, 0);
+        }
     }
     delete $$et{DOC_NUM};
     return 1;
@@ -1859,6 +1927,7 @@ sub ProcessRAF($$)
     my ($buff, $jpeg, $warn, $offset);
 
     my $raf = $$dirInfo{RAF};
+    my $base = $$dirInfo{Base} || 0;
     $raf->Read($buff,0x70) == 0x70    or return 0;
     $buff =~ /^FUJIFILM/              or return 0;
     # get position and size of M-RAW header and jpeg preview
@@ -1866,13 +1935,13 @@ sub ProcessRAF($$)
     my ($jpos, $jlen) = unpack('x84NN', $buff);
     $jpos & 0x8000                   and return 0;
     if ($jpos) {
-        $raf->Seek($jpos, 0)              or return 0;
+        $raf->Seek($jpos+$base, 0)        or return 0;
         $raf->Read($jpeg, $jlen) == $jlen or return 0;
     }
     SetByteOrder('MM');
-    $et->SetFileType();
+    $et->SetFileType() unless $$et{DOC_NUM};
     my $tbl = GetTagTable('Image::ExifTool::FujiFilm::RAFHeader');
-    $et->ProcessDirectory({ DataPt => \$buff, DirName => 'RAFHeader' }, $tbl);
+    $et->ProcessDirectory({ DataPt => \$buff, DirName => 'RAFHeader', Base => $base }, $tbl);
     
     # extract information from embedded JPEG
     my %dirInfo = (
@@ -1880,21 +1949,22 @@ sub ProcessRAF($$)
         RAF    => File::RandomAccess->new(\$jpeg),
     );
     if ($jpos) {
-        $$et{BASE} += $jpos;
+        $$et{BASE} += $jpos + $base;
         my $ok = $et->ProcessJPEG(\%dirInfo);
-        $$et{BASE} -= $jpos;
+        $$et{BASE} -= $jpos + $base;
         $et->FoundTag('PreviewImage', \$jpeg) if $ok;
     }
     # extract information from Fuji RAF and TIFF directories
     my ($rafNum, $ifdNum) = ('','');
     foreach $offset (0x48, 0x5c, 0x64, 0x78, 0x80) {
         last if $jpos and $offset >= $jpos;
-        unless ($raf->Seek($offset, 0) and $raf->Read($buff, 8)) {
+        unless ($raf->Seek($offset+$base, 0) and $raf->Read($buff, 8)) {
             $warn = 1;
             last;
         }
         my ($start, $len) = unpack('N2',$buff);
         next unless $start;
+        $start += $base;
         if ($offset == 0x64 or $offset == 0x80) {
             # parse FujiIFD directory
             %dirInfo = (
