@@ -7,6 +7,7 @@
 #
 # References:   1) https://github.com/gopro/gpmf-parser
 #               2) https://github.com/stilldavid/gopro-utils
+#               3) https://github.com/gopro/gpmf-parser
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::GoPro;
@@ -16,7 +17,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::QuickTime;
 
-$VERSION = '1.03';
+$VERSION = '1.14';
 
 sub ProcessGoPro($$$);
 sub ProcessString($$$);
@@ -59,6 +60,8 @@ my %addUnits = (
     PrintConv => 'Image::ExifTool::GoPro::AddUnits($self, $val, $tag)',
 );
 
+my %noYes = ( N => 'No', Y => 'Yes' );
+
 # Tags found in the GPMF box of Hero6 mp4 videos (ref PH), and
 # the gpmd-format timed metadata of Hero5 and Hero6 videos (ref 1)
 %Image::ExifTool::GoPro::GPMF = (
@@ -72,13 +75,19 @@ my %addUnits = (
         let me know if you discover the meaning of any of these unknown tags. See
         L<https://github.com/gopro/gpmf-parser> for details about this format.
     },
+    ABSC => 'AutoBoostScore', #3
     ACCL => { #2 (gpmd)
         Name => 'Accelerometer',
         Notes => 'accelerator readings in m/s2',
         Binary => 1,
     },
+  # ANGX (GPMF-GEOC) - seen -0.05 (fmt d, Max)
+  # ANGY (GPMF-GEOC) - seen 179.9 (fmt d, Max)
+  # ANGZ (GPMF-GEOC) - seen 0.152 (fmt d, Max)
     ALLD => 'AutoLowLightDuration', #1 (gpmd) (untested)
-  # APTO (GPMF) - seen: 'RAW', 'DYNM' (fmt c)
+    APTO => 'AudioProtuneOption', #3
+    ARUW => 'AspectRatioUnwarped', #3
+    ARWA => 'AspectRatioWarped', #3
     ATTD => { #PH (Karma)
         Name => 'Attitude',
         # UNIT=s,rad,rad,rad,rad/s,rad/s,rad/s,
@@ -93,8 +102,10 @@ my %addUnits = (
         # SCAL=1000 1 1 1 1
         Binary => 1,
     },
+    AUBT => { Name => 'AudioBlueTooth', PrintConv => \%noYes }, #3
     AUDO => 'AudioSetting', #PH (GPMF - seen: 'WIND', fmt c)
-  # AUPT (GPMF) - seen: 'N','Y' (fmt c)
+    AUPT => { Name => 'AutoProtune', PrintConv => \%noYes },
+    BITR => 'BitrateSetting', #3
     BPOS => { #PH (Karma)
         Name => 'Controller',
         Unknown => 1,
@@ -105,16 +116,27 @@ my %addUnits = (
     },
   # BRID (GPMF) - seen: 0 (fmt B)
   # BROD (GPMF) - seen: 'ASK','' (fmt c)
+  # CALH (GPMF-GEOC) - seen 3040 (fmt L, Max)
+  # CALW (GPMF-GEOC) - seen 4056 (fmt L, Max)
     CASN => 'CameraSerialNumber', #PH (GPMF - seen: 'C3221324545448', fmt c)
-  # CINF (GPMF) - seen: 0x67376be7709bc8876a8baf3940908618, 0xe230988539b30cf5f016627ae8fc5395 (fmt B) (Camera INFormation?)
-  # CMOD (GPMF) - seen: 12,13,17 [13 time-laps video, 17 JPEG] (fmt B)
-    CYTS => { #PH (Karma)
-        Name => 'CoyoteStatus',
-        # UNIT=s,,,,,rad,rad,rad,,
-        # TYPE=LLLLLfffBB
-        # SCAL=1000 1 1 1 1 1 1 1 1 1
-        Binary => 1,
+    CDAT => { #3
+        Name => 'CreationDate',
+        Groups => { 2 => 'Time' },
+        RawConv => 'ConvertUnixTime($val)',
+        PrintConv => '$self->ConvertDateTime($val)',
     },
+    CDTM => 'CaptureDelayTimer', #3
+  # CINF (GPMF) - seen: 0x67376be7709bc8876a8baf3940908618, 0xe230988539b30cf5f016627ae8fc5395,
+  #                     0x8bcbe424acc5b37d7d77001635198b3b (fmt B) (Camera INFormation?)
+    CLDP => { Name => 'ClassificationDataPresent', PrintConv => \%noYes },
+  # CLKC (GPMF) - seen: 0 (fmt L)
+  # CLKS (GPMF) - seen: 2 (fmt B)
+  # CMOD (GPMF) - seen: 12,13,17 [12 360 video, 13 time-laps video, 17 JPEG] (fmt B) - CameraMode (ref 3)
+  # CPID (GPMF) - seen: '1194885996 3387225026 733916448 2433577768' (fmt L)
+  # CPIN (GPMF) - seen: 1
+    CPIN => 'ChapterNumber',
+  # CRTX (GPMF-BACK/FRNT) - double[1]
+  # CRTY (GPMF-BACK/FRNT) - double[1]
     CSEN => { #PH (Karma)
         Name => 'CoyoteSense',
         # UNIT=s,rad/s,rad/s,rad/s,g,g,g,,,,
@@ -122,24 +144,42 @@ my %addUnits = (
         # SCAL=1000 1 1 1 1 1 1 1 1 1 1
         Binary => 1,
     },
+    CTRL => 'ControlLevel', #3
+    CYTS => { #PH (Karma)
+        Name => 'CoyoteStatus',
+        # UNIT=s,,,,,rad,rad,rad,,
+        # TYPE=LLLLLfffBB
+        # SCAL=1000 1 1 1 1 1 1 1 1 1
+        Binary => 1,
+    },
     DEVC => { #PH (gpmd,GPMF, fmt \0)
         Name => 'DeviceContainer',
         SubDirectory => { TagTable => 'Image::ExifTool::GoPro::GPMF' },
+        # (Max) DVID=1,DVNM='Global Settings',VERS,FMWR,LINF,CINF,CASN,MINF,MUID,CMOD,MTYP,OREN,
+        #       DZOM,DZST,SMTR,PRTN,PTWB,PTSH,PTCL,EXPT,PIMX,PIMN,PTEV,RATE,SROT,ZFOV,VLTE,VLTA,
+        #       EISE,EISA,AUPT,AUDO,BROD,BRID,PVUL,PRJT,SOFF
+        # (Max) DVID='GEOC',DVNM='Geometry Calibrations',SHFX,SHFY,SHFZ,ANGX,ANGY,ANGZ,CALW,CALH
+        # (Max) DVID='BACK',DVNM='Back Lens',KLNS,CTRX,CTRY,MFOV,SFTR
+        # (Max) DVID='FRNT',DVNM='Front Lens',KLNS,CTRX,CTRY,MFOV,SFTR
+        # (Max) DVID='HLMT',DVNM='Highlights'
     },
-  # DVID (GPMF) - DeviceID; seen: 1 (fmt L), HLMT (fmt F)
+  # DNSC (GPMF) - seen: 'HIGH' (fmt c)
+    DUST => 'DurationSetting', #3
+  # DVID (GPMF) - DeviceID; seen: 1 (fmt L), HLMT (fmt F), GEOC (fmt F), 'BACK' (fmt F, Max)
     DVID => { Name => 'DeviceID', Unknown => 1 }, #2 (gpmd)
-  # DVNM (GPMF) seen: 'Video Global Settings' (fmt c), 'Highlights' (fmt c)
+  # DVNM (GPMF) seen: 'Video Global Settings' (fmt c), 'Highlights' (fmt c), 'Geometry Calibrations' (Max)
   # DVNM (gpmd) seen: 'Camera' (Hero5), 'Hero6 Black' (Hero6), 'GoPro Karma v1.0' (Karma)
     DVNM => 'DeviceName', #PH (n/c)
     DZOM => { #PH (GPMF - seen: 'Y', fmt c)
-        Name => 'DigitalZoom',
-        PrintConv => { N => 'No', Y => 'Yes' },
+        Name => 'DigitalZoomOn',
+        PrintConv => \%noYes,
     },
-  # DZST (GPMF) - seen: 0 (fmt L) (something to do with digital zoom maybe?)
-    EISA => { #PH (GPMF) - seen: 'Y','N', 'HS EIS' (fmt c) [N was for a time-lapse video]
+    DZMX => 'DigitalZoomAmount', #3
+    DZST => 'DigitalZoom', #3
+    EISA => { #PH (GPMF) - seen: 'Y','N','HS EIS','N/A' (fmt c) [N was for a time-lapse video]
         Name => 'ElectronicImageStabilization',
     },
-  # EISE (GPMF) - seen: 'Y' (fmt c)
+    EISE => { Name => 'ElectronicStabilizationOn', PrintConv => \%noYes }, #3
     EMPT => { Name => 'Empty', Unknown => 1 }, #2 (gpmd)
     ESCS => { #PH (Karma)
         Name => 'EscapeStatus',
@@ -149,7 +189,7 @@ my %addUnits = (
         Unknown => 1,
         %addUnits,
     },
-  # EXPT (GPMF) - seen: '', 'AUTO' (fmt c)
+    EXPT => 'ExposureType', #3
     FACE => 'FaceDetected', #PH (gpmd)
     FCNM => 'FaceNumbers', #PH (gpmd) (faces counted per frame, ref 1)
     FMWR => 'FirmwareVersion', #PH (GPMF - seen: HD6.01.01.51.00, fmt c)
@@ -173,10 +213,20 @@ my %addUnits = (
     },
     GPS5 => { #2 (gpmd)
         Name => 'GPSInfo',
+        # UNIT=deg,deg,m,m/s,m/s
+        # TYPE=l
         # SCAL=10000000,10000000,1000,1000,100
         RawConv => '$val', # necessary to use scaled value instead of raw data as subdir data
         SubDirectory => { TagTable => 'Image::ExifTool::GoPro::GPS5' },
     },
+    GPS9 => { #PH (gpmd, Hero 13)
+        Name => 'GPSInfo9',
+        # UNIT=deg,deg,m,m/s,m/s,,s,,
+        # TYPE=lllllllSS
+        # SCAL=10000000 10000000 1000 1000 100 1 1000 100 1
+        RawConv => '$val', # necessary to use scaled value instead of raw data as subdir data
+        SubDirectory => { TagTable => 'Image::ExifTool::GoPro::GPS9' },
+    },
     GPSF => { #2 (gpmd)
         Name => 'GPSMeasureMode',
         PrintConv => {
@@ -201,7 +251,10 @@ my %addUnits = (
         Notes => 'gyroscope readings in rad/s',
         Binary => 1,
     },
+    HCLT => 'HorizonControl', #3
+    HDRV => { Name => 'HDRVideo', PrintConv => \%noYes }, #3/PH (NC)
   # HFLG (APP6) - seen: 0
+    HSGT => 'HindsightSettings', #3
     ISOE => 'ISOSpeeds', #PH (gpmd)
     ISOG => { #2 (gpmd)
         Name => 'ImageSensorGain',
@@ -215,7 +268,9 @@ my %addUnits = (
         RawConv => '$val', # necessary to use scaled value instead of raw data as subdir data
         SubDirectory => { TagTable => 'Image::ExifTool::GoPro::KBAT' },
     },
-  # LINF (GPMF) - seen: LAJ7061916601668, C3341326002180 (fmt c) (Lens INFormation?)
+  # KLNS (GPMF-BACK/FRNT) - double[5] (fmt d, Max)
+  # LINF (GPMF) - seen: LAJ7061916601668,C3341326002180,C33632245450981 (fmt c) (Lens INFormation?)
+  # LMOD (GPMF) - seen: 'NONE' (fmt F)
     LNED => { #PH (Karma)
         Name => 'LocalPositionNED',
         # UNIT=s,m,m,m,m/s,m/s,m/s
@@ -224,13 +279,20 @@ my %addUnits = (
         Binary => 1,
     },
     MAGN => 'Magnetometer', #1 (gpmd) (units of uT)
+    MAPX => 'MappingXCoefficients', #3
+    MAPY => 'MappingYCoefficients', #3
+  # MFOV (GPMF-BACK/FRNT) - seen: 100 (fmt d, Max)
     MINF => { #PH (GPMF - seen: HERO6 Black, fmt c)
         Name => 'Model',
         Groups => { 2 => 'Camera' },
         Description => 'Camera Model Name',
     },
-  # MTYP (GPMF) - seen: 0,1,11 [1 for time-lapse video, 11 for JPEG] (fmt B)
-  # MUID (GPMF) - seen: 3882563431 2278071152 967805802 411471936 0 0 0 0 (fmt L)
+    MMOD => 'MediaMode', #3
+  # MTYP (GPMF) - seen: 0,1,5,11 [1 for time-lapse video, 5 for 360 video, 11 for JPEG] (fmt B) - MediaType (ref 3)
+    MUID => { Name => 'MediaUID', ValueConv => 'join("-", unpack("H8H4H4H4H12", $val))' },
+    MXCF => 'MappingXMode', #3
+    MYCF => 'MappingYMode', #3
+    ORDP => { Name => 'OrientationDataPresent', PrintConv => \%noYes }, #3
     OREN => { #PH (GPMF - seen: 'U', fmt c)
         Name => 'AutoRotation',
         PrintConv => {
@@ -239,15 +301,20 @@ my %addUnits = (
             A => 'Auto', # (NC)
         },
     },
-    # (most of the "P" tags are ProTune settings - PH)
+    # (most of the "P" tags are Protune settings - PH)
     PHDR => 'HDRSetting', #PH (APP6 - seen: 0)
+    PIMD => 'ProtuneISOMode', #3
     PIMN => 'AutoISOMin', #PH (GPMF - seen: 100, fmt L)
     PIMX => 'AutoISOMax', #PH (GPMF - seen: 1600, fmt L)
+    POLY => 'PolynomialCoefficients', #3
   # PRAW (APP6) - seen: 0, 'N', 'Y' (fmt c)
+  # PRCN (GPMF) - seen: 65 zeros (fmt B)
     PRES => 'PhotoResolution', #PH (APP6 - seen: '12MP_W')
-  # PRJT (APP6) - seen: 'GPRO' (fmt F, Hero8)
+    PRJT => 'LensProjection', #3
+  # PRNA (GPMF) - seen 10 (fmt B) - PresetIDs (ref 3)
+  # PRNU (GPMF) - seen 0 (fmt B) - PresetIDs (ref 3)
     PRTN => { #PH (GPMF - seen: 'N', fmt c)
-        Name => 'ProTune',
+        Name => 'Protune',
         PrintConv => {
             N => 'Off',
             Y => 'On', # (NC)
@@ -257,7 +324,10 @@ my %addUnits = (
     PTEV => 'ExposureCompensation', #PH (GPMF - seen: '0.0', fmt c)
     PTSH => 'Sharpness', #PH (GPMF - seen: 'HIGH', fmt c)
     PTWB => 'WhiteBalance', #PH (GPMF - seen: 'AUTO', fmt c)
-  # PVUL (APP6) - seen: 'F' (fmt c, Hero8)
+  # PVUL (APP6) - seen: 'F' (fmt c, Hero8, Max)
+    PWPR => 'PowerProfile', #3
+    PYCF => 'PolynomialPower', #3
+    RAMP => 'SpeedRampSetting', #3
     RATE => 'Rate', #PH (GPMF - seen: '0_5SEC', fmt c; APP6 - seen: '4_1SEC')
     RMRK => { #2 (gpmd)
         Name => 'Comments',
@@ -267,6 +337,7 @@ my %addUnits = (
         Name => 'ScaleFactor',
         Unknown => 1,
     },
+    SCAP => { Name => 'ScheduleCapture', PrintConv => \%noYes }, #3
     SCPR => { #PH (Karma) [stream was empty]
         Name => 'ScaledPressure',
         # UNIT=s,Pa,Pa,degC
@@ -274,6 +345,11 @@ my %addUnits = (
         # SCAL=1000 0.00999999977648258 0.00999999977648258 100
         %addUnits,
     },
+    SCTM => 'ScheduleCaptureTime', #3 (seconds since UTC midnight)
+  # SFTR (GPMF-BACK/FRNT) - seen 0.999,1.00004 (fmt d, Max)
+  # SHFX (GPMF-GEOC) - seen 22.92 (fmt d, Max)
+  # SHFY (GPMF-GEOC) - seen 0.123 (fmt d, Max)
+  # SHFZ (GPMF-GEOC) - seen 36.06 (fmt d, Max)
     SHUT => { #2 (gpmd)
         Name => 'ExposureTimes',
         PrintConv => q{
@@ -294,8 +370,9 @@ my %addUnits = (
         Unknown => 1,
         ValueConv => '$self->Decode($val, "Latin")',
     },
-  # SMTR (GPMF) - seen: 'N' (fmt c)
-  # SOFF (APP6) - seen: 0 (fmt L, Hero8)
+    SMTR => { Name => 'SpotMeter', PrintConv => \%noYes }, #3
+  # SOFF (APP6) - seen: 0 (fmt L, Hero8, Max)
+    SROT => 'SensorReadoutTime', #3
     STMP => { #1 (gpmd)
         Name => 'TimeStamp',
         ValueConv => '$val / 1e6',
@@ -333,12 +410,19 @@ my %addUnits = (
   # TOCK => { Name => 'OutTime', Unknown => 1, ValueConv => '$val/1000' }, #1 (gpmd)
     TSMP => { Name => 'TotalSamples', Unknown => 1 }, #2 (gpmd)
     TYPE => { Name => 'StructureType', Unknown => 1 }, #2 (gpmd,GPMF - eg 'LLLllfFff', fmt c)
+    TZON => { # (GPMF) - seen: 60 (fmt s)
+        Name => 'TimeZone',
+        PrintConv => 'Image::ExifTool::TimeZoneString($val)',
+    },
     UNIT => { #2 (gpmd) alternative units
         Name => 'Units',
         Unknown => 1,
         ValueConv => '$self->Decode($val, "Latin")',
     },
-  # VERS (APP6) - seen: '7 6 4' (fmt B, Hero8)
+    VERS => {
+        Name => 'MetadataVersion',
+        PrintConv => '$val =~ tr/ /./; $val',
+    },
     VFOV => { #PH (GPMF - seen: 'W', fmt c)
         Name => 'FieldOfView',
         PrintConv => {
@@ -348,6 +432,7 @@ my %addUnits = (
         },
     },
   # VLTA (GPMF) - seen: 78 ('N') (fmt B -- wrong format?)
+    VFPS => { Name => 'VideoFrameRate', PrintConv => '$val=~s( )(/);$val' }, #PH (GPMF, fmt L)
     VFRH => { #PH (Karma)
         Name => 'VisualFlightRulesHUD',
         BinaryData => 1,
@@ -355,19 +440,52 @@ my %addUnits = (
         # TYPE=ffffsS
     },
   # VLTE (GPMF) - seen: 'Y','N' (fmt c)
+    VRES => { Name => 'VideoFrameSize', PrintConv => '$val=~s/ /x/;$val' }, #PH (GPMF, fmt L)
     WBAL => 'ColorTemperatures', #PH (gpmd)
     WRGB => { #PH (gpmd)
         Name => 'WhiteBalanceRGB',
         Binary => 1,
     },
-  # ZFOV (APP6) - seen: 148.34 (fmt f, Hero8)
+    ZFOV => 'DiagonalFieldOfView', #3
+    ZMPL => 'ZoomScaleNormalization', #3
+#
+# the following ref forum12825
+#
+    MUID => {
+        Name => 'MediaUniqueID',
+        PrintConv => q{
+            my @a = split ' ', $val;
+            $_ = sprintf('%.8x',$_) foreach @a;
+            return join('', @a);
+        },
+    },
+    MTRX => 'AccelerometerMatrix',
+    ORIN => 'InputOrientation',
+    ORIO => 'OutputOrientation',
+    UNIF => 'InputUniformity',
+    SROT => 'SensorReadoutTime',
+    # the following are ref https://exiftool.org/forum/index.php?topic=15517.0
+    CORI => { Name => 'CameraOrientation', Binary => 1, Notes => 'quaternions 0-1' },
+    AALP => { Name => 'AudioLevel', Notes => 'dBFS' },
+    GPSA => 'GPSAltitudeSystem', # (eg. 'MSLV')
+    GRAV => { Name => 'GravityVector', Binary => 1 },
+    HUES => 'PrediminantHue',
+    IORI => { Name => 'ImageOrientation', Binary => 1, Notes => 'quaternions 0-1' },
+    # LRVO - ? Part of LRV Frame Skip
+    # LRVS - ? Part of LRV Frame Skip
+    # LSKP - LRV Frame Skip
+    # MSKP - MRV Frame Skip
+    MWET => 'MicrophoneWet',
+    SCEN => 'SceneClassification', # (SNOW,URBA,INDO,WATR,VEGE,BEAC + probability)
+    WNDM => 'WindProcessing',
+    YAVG => 'LumaAverage',
 );
 
-# GoPro GPS5 tags (ref 2) (Hero5,Hero6)
+# GoPro GPS5 tags (ref 2) (Hero5,Hero6,Hero9)
 %Image::ExifTool::GoPro::GPS5 = (
     PROCESS_PROC => \&ProcessString,
     GROUPS => { 1 => 'GoPro', 2 => 'Location' },
-    VARS => { HEX_ID => 0, ID_LABEL => 'Index' },
+    VARS => { ID_FMT => 'dec', ID_LABEL => 'Index' },
     0 => { # (unit='deg')
         Name => 'GPSLatitude',
         PrintConv => 'Image::ExifTool::GPS::ToDMS($self, $val, 1, "N")',
@@ -380,15 +498,72 @@ my %addUnits = (
         Name => 'GPSAltitude',
         PrintConv => '"$val m"',
     },
-    3 => 'GPSSpeed',   # (unit='m/s')
-    4 => 'GPSSpeed3D', # (unit='m/s')
+    3 => {
+        Name => 'GPSSpeed',
+        Notes => 'stored as m/s but converted to km/h when extracted',
+        ValueConv => '$val * 3.6',
+    },
+    4 => {
+        Name => 'GPSSpeed3D',
+        Notes => 'stored as m/s but converted to km/h when extracted',
+        ValueConv => '$val * 3.6',
+    },
+);
+
+# GoPro GPS9 tags (Hero13)
+%Image::ExifTool::GoPro::GPS9 = (
+    PROCESS_PROC => \&ProcessString,
+    GROUPS => { 1 => 'GoPro', 2 => 'Location' },
+    VARS => { ID_FMT => 'dec', ID_LABEL => 'Index' },
+    0 => { # (unit='deg')
+        Name => 'GPSLatitude',
+        PrintConv => 'Image::ExifTool::GPS::ToDMS($self, $val, 1, "N")',
+    },
+    1 => { # (unit='deg')
+        Name => 'GPSLongitude',
+        PrintConv => 'Image::ExifTool::GPS::ToDMS($self, $val, 1, "E")',
+    },
+    2 => { # (unit='m')
+        Name => 'GPSAltitude',
+        PrintConv => '"$val m"',
+    },
+    3 => {
+        Name => 'GPSSpeed',
+        Notes => 'stored as m/s but converted to km/h when extracted',
+        ValueConv => '$val * 3.6',
+    },
+    4 => {
+        Name => 'GPSSpeed3D',
+        Notes => 'stored as m/s but converted to km/h when extracted',
+        ValueConv => '$val * 3.6',
+    },
+    5 => { # days since 2000
+        Name => 'GPSDays',
+        RawConv => '$$self{GPSDays} = $val; undef',
+        Hidden => 1,
+    },
+    6 => { # seconds of date/time
+        Name => 'GPSDateTime',
+        Groups => { 2 => 'Time' },
+        # (10957 days from Jan 1 1970 to Jan 1 2000)
+        RawConv => 'ConvertUnixTime(($$self{GPSDays} + 10957) * 24 * 3600 + $val, undef, 3)',
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+    7 => 'GPSDOP',
+    8 => {
+        Name => 'GPSMeasureMode', #PH (NC)
+        PrintConv => {
+            2 => '2-Dimensional Measurement',
+            3 => '3-Dimensional Measurement',
+        },
+    },
 );
 
 # GoPro GPRI tags (ref PH) (Karma)
 %Image::ExifTool::GoPro::GPRI = (
     PROCESS_PROC => \&ProcessString,
     GROUPS => { 1 => 'GoPro', 2 => 'Location' },
-    VARS => { HEX_ID => 0, ID_LABEL => 'Index' },
+    VARS => { ID_FMT => 'dec', ID_LABEL => 'Index' },
     0 => { # (unit='s')
         Name => 'GPSDateTimeRaw',
         Groups => { 2 => 'Time' },
@@ -420,7 +595,7 @@ my %addUnits = (
 %Image::ExifTool::GoPro::GLPI = (
     PROCESS_PROC => \&ProcessString,
     GROUPS => { 1 => 'GoPro', 2 => 'Location' },
-    VARS => { HEX_ID => 0, ID_LABEL => 'Index' },
+    VARS => { ID_FMT => 'dec', ID_LABEL => 'Index' },
     0 => { # (unit='s')
         Name => 'GPSDateTime',
         Groups => { 2 => 'Time' },
@@ -451,7 +626,7 @@ my %addUnits = (
 %Image::ExifTool::GoPro::KBAT = (
     PROCESS_PROC => \&ProcessString,
     GROUPS => { 1 => 'GoPro', 2 => 'Camera' },
-    VARS => { HEX_ID => 0, ID_LABEL => 'Index' },
+    VARS => { ID_FMT => 'dec', ID_LABEL => 'Index' },
     NOTES => 'Battery status information found in GoPro Karma videos.',
      0 => { Name => 'BatteryCurrent',  PrintConv => '"$val A"' },
      1 => { Name => 'BatteryCapacity', PrintConv => '"$val Ah"' },
@@ -549,7 +724,7 @@ sub ScaleValues($$)
 sub AddUnits($$$)
 {
     my ($et, $val, $tag) = @_;
-    if ($et and $$et{TAG_EXTRA}{$tag} and $$et{TAG_EXTRA}{$tag}{Units}) {
+    if ($et and $$et{TAG_EXTRA}{$tag}{Units}) {
         my $u = $$et{TAG_EXTRA}{$tag}{Units};
         $u = [ $u ] unless ref $u eq 'ARRAY';
         my @a = split ' ', $val;
@@ -575,21 +750,60 @@ sub ProcessString($$$)
     my @list = ref $$dataPt eq 'ARRAY' ? @{$$dataPt} : ( $$dataPt );
     my ($string, $val);
     $et->VerboseDir('GoPro structure');
+    my $docNum = $$et{DOC_NUM};
+    my $subDoc = 0;
     foreach $string (@list) {
         my @val = split ' ', $string;
         my $i = 0;
         foreach $val (@val) {
             $et->HandleTag($tagTablePtr, $i, $val);
-            $$tagTablePtr{++$i} or $i = 0;
+            next if $$tagTablePtr{++$i};
+            # increment subdoc for records stored as string of values (eg. GPS5)
+            $i = 0;
+            ++$subDoc;
+            $$et{DOC_NUM} = "$docNum-$subDoc";
+        }
+        if ($i) {
+            # increment subdoc for records stored as array of strings (eg. GPS9)
+            ++$subDoc;
+            $$et{DOC_NUM} = "$docNum-$subDoc";
         }
     }
+    $$et{DOC_NUM} = $docNum;
     return 1;
 }
 
+#------------------------------------------------------------------------------
+# Process "GP\x06\0" records in MP4 'mdat' atom
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref (RAF and DirLen)
+# Returns: size of GoPro record, or 0 on error
+sub ProcessGP6($$)
+{
+    my ($et, $dirInfo) = @_;
+    my $raf = $$dirInfo{RAF};
+    my $len = $$dirInfo{DirLen};
+    my $buff;
+    while ($len > 16) {
+        $raf->Read($buff, 16) == 16 or last;
+        my ($tag, $size) = unpack('a4N', $buff);
+        last if $size + 16 > $len or $buff !~ /^GP..\0/s;
+        $raf->Read($buff, $size) == $size or last;
+        if ($buff =~ /^DEVC/) {
+            $$et{DOC_NUM} = ++$$et{DOC_COUNT};
+            my $tagTbl = GetTagTable('Image::ExifTool::GoPro::GPMF');
+            ProcessGoPro($et, { DataPt => \$buff, DataPos => $raf->Tell()-$size }, $tagTbl);
+        }
+        $len -= $size + 16;
+    }
+    delete $$et{DOC_NUM};
+    return $$dirInfo{DirLen} - $len;
+}
+
 #------------------------------------------------------------------------------
 # Process GoPro metadata (gpmd samples, GPMF box, or APP6) (ref PH/1/2)
 # Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
 # Returns: 1 on success
+# - with hack to check for encrypted text in gpmd data (Rove Stealth 4K)
 sub ProcessGoPro($$$)
 {
     my ($et, $dirInfo, $tagTablePtr) = @_;
@@ -602,6 +816,7 @@ sub ProcessGoPro($$$)
     my ($size, $type, $unit, $scal, $setGroup0);
 
     $et->VerboseDir($$dirInfo{DirName} || 'GPMF', undef, $dirEnd-$pos) if $verbose;
+    $$et{FoundEmbedded} = 1;
     if ($pos) {
         my $parent = $$dirInfo{Parent};
         $setGroup0 = $$et{SET_GROUP0} = 'APP6' if $parent and $parent eq 'APP6';
@@ -612,9 +827,16 @@ sub ProcessGoPro($$$)
 
     for (; $pos+8<=$dirEnd; $pos+=($size+3)&0xfffffffc) {
         my ($tag,$fmt,$len,$count) = unpack("x${pos}a4CCn", $$dataPt);
+        if ($tag =~ /[^-_a-zA-Z0-9 ]/) {
+            $et->Warn('Unrecognized GoPro record') unless $tag eq "\0\0\0\0";
+            last;
+        }
         $size = $len * $count;
         $pos += 8;
-        last if $pos + $size > $dirEnd;
+        if ($pos + $size > $dirEnd) {
+            $et->Warn('Truncated GoPro record');
+            last;
+        }
         my $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
         last if $tag eq "\0\0\0\0";     # stop at null tag
         next unless $size or $verbose;  # don't save empty values unless verbose
@@ -659,6 +881,7 @@ sub ProcessGoPro($$$)
         ScaleValues($val, $scal) if $scal and $tag ne 'SCAL' and $pos+$size+3>=$dirEnd;
         my $key = $et->HandleTag($tagTablePtr, $tag, $val,
             DataPt  => $dataPt,
+            DataPos => $$dirInfo{DataPos},
             Base    => $base,
             Start   => $pos,
             Size    => $size,
@@ -692,7 +915,7 @@ metadata from GoPro MP4 videos.
 
 =head1 AUTHOR
 
-Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2025, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -705,6 +928,8 @@ under the same terms as Perl itself.
 
 =item L<https://github.com/stilldavid/gopro-utils>
 
+=item L<https://github.com/gopro/gpmf-parser>
+
 =back
 
 =head1 SEE ALSO
