@@ -260,15 +260,9 @@ sub ProcessJPEG_HDR($$$);
         Condition => '$$valPt =~ /^UNICODE\0/',
         Notes => 'PhotoStudio Unicode comment',
       }, {
-        Name => 'HDRGainCurve', #PH (NC)
+        Name => 'HDRGainInfo', #PH (NC)
         Condition => '$$valPt =~ /^AROT\0\0.{4}/s',
-        Groups => { 1 => 'APP10', 2 => 'Image' },
-        ValueConv => q{
-            my $n = unpack('x6N', $val);
-            return '<truncated AROT data>' if length($val)-6 < $n * 4;
-            my $str = join ' ', unpack("x10V$n", $val);
-            return \$str;
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::JPEG::HDRGainInfo' },
     }],
     APP11 => [{
         Name => 'JPEG-HDR',
@@ -347,6 +341,9 @@ sub ProcessJPEG_HDR($$$);
             $$valPt =~ /~\0\x04\0zmie~\0\0\x0a.{8}[\x10\x18]\x08$/s
         },
         SubDirectory => { TagTable => 'Image::ExifTool::MIE::Main' },
+      }, {
+        Name => 'MPF',
+        SubDirectory => { TagTable => 'Image::ExifTool::MPF::Main' },
       }, {
         Name => 'Samsung',
         Condition => '$$valPt =~ /QDIOBS$/',
@@ -382,6 +379,21 @@ sub ProcessJPEG_HDR($$$);
     }],
 );
 
+# HDR gain information (ref PH)
+%Image::ExifTool::JPEG::HDRGainInfo = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 0 => 'APP10', 1 => 'AROT', 2 => 'Image' },
+    6 => {
+        Name => 'HDRGainCurveSize',
+        Format => 'int32u',
+    },
+    10 => {
+        Name => 'HDRGainCurve', # (NC)
+        Format => 'int32uRev[$val{6}]',
+        Binary => 1,
+    },
+);
+
 # JPS APP3 segment (ref http://paulbourke.net/stereographics/stereoimage/)
 %Image::ExifTool::JPEG::JPS = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
@@ -553,7 +565,7 @@ sub ProcessJPEG_HDR($$$);
 # APP9 Media Jukebox segment (ref PH)
 %Image::ExifTool::JPEG::MediaJukebox = (
     GROUPS => { 0 => 'XML', 1 => 'MediaJukebox', 2 => 'Image' },
-    VARS => { NO_ID => 1 },
+    VARS => { ID_FMT => 'none' },
     NOTES => 'Tags found in the XML metadata of the APP9 "Media Jukebox" segment.',
     Date => {
         Groups => { 2 => 'Time' },
