@@ -14,7 +14,7 @@ use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::XMP;
 use Image::ExifTool::ZIP;
 
-$VERSION = '1.08';
+$VERSION = '1.09';
 
 # test for recognized OOXML document extensions
 my %isOOXML = (
@@ -54,7 +54,7 @@ my @vectorVals;
 %Image::ExifTool::OOXML::Main = (
     GROUPS => { 0 => 'XML', 1 => 'XML', 2 => 'Document' },
     PROCESS_PROC => \&Image::ExifTool::XMP::ProcessXMP,
-    VARS => { NO_ID => 1 },
+    VARS => { ID_FMT => 'none' },
     NOTES => q{
         The Office Open XML (OOXML) format was introduced with Microsoft Office 2007
         and is used by file types such as DOCX, PPTX and XLSX.  These are
@@ -360,12 +360,12 @@ sub ProcessDOCX($$)
         }
         # process XML files (docProps/app.xml, docProps/core.xml, docProps/custom.xml)
         my %dirInfo = (
-            DataPt => \$buff,
-            DirLen => length $buff,
-            DataLen => length $buff,
-            XMPParseOpts => {
-                FoundProc => \&FoundTag,
-            },
+            DataPt   => \$buff,
+            DirLen   => length $buff,
+            # (skip over XML header if it exists)
+            DirStart => ($buff =~ /<\?xml\s+.*?\?>/g ? pos($buff) : 0),
+            DataLen  => length $buff,
+            XMPParseOpts => { FoundProc => \&FoundTag },
         );
         $et->ProcessDirectory(\%dirInfo, $tagTablePtr);
         undef $buff;    # (free memory now)
@@ -395,7 +395,7 @@ archives of XML files.
 
 =head1 AUTHOR
 
-Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2025, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
