@@ -19,8 +19,9 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::XMP;
+use Image::ExifTool::GPS;
 
-$VERSION = '1.02';
+$VERSION = '1.15';
 
 sub ExtractObject($$;$);
 sub Get24u($$);
@@ -36,11 +37,16 @@ my %readProc = (
     0x108 => \&GetDouble,
 );
 
+# recognize different types of PLIST files based on certain tags
+my %plistType = (
+    adjustmentBaseVersion => 'AAE',
+);
+
 # PLIST tags (generated on-the-fly for most tags)
 %Image::ExifTool::PLIST::Main = (
     PROCESS_PROC => \&ProcessPLIST,
     GROUPS => { 0 => 'PLIST', 1 => 'XML', 2 => 'Document' },
-    VARS => { LONG_TAGS => 4 },
+    VARS => { LONG_TAGS => 12 },
     NOTES => q{
         Apple Property List tags.  ExifTool reads both XML and binary-format PLIST
         files, and will extract any existing tags even if they aren't listed below.
@@ -64,7 +70,7 @@ my %readProc = (
         Description => 'Date/Time Original',
         Groups => { 2 => 'Time' },
         # Sony uses a "real" here -- number of days since Dec 31, 1899
-        ValueConv => 'IsFloat($val) ? ConvertUnixTime(($val - 25569) * 24 * 3600 + 0.5) : $val',
+        ValueConv => 'IsFloat($val) ? ConvertUnixTime(($val - 25569) * 24 * 3600) : $val',
         PrintConv => '$self->ConvertDateTime($val)',
     },
     'MetaDataList//Duration' => {
@@ -75,23 +81,46 @@ my %readProc = (
     'MetaDataList//Geolocation/Latitude' => {
         Name => 'GPSLatitude',
         Groups => { 2 => 'Location' },
-        PrintConv => q{
-            require Image::ExifTool::GPS;
-            Image::ExifTool::GPS::ToDMS($self, $val, 1, 'N');
-        },
+        PrintConv => 'Image::ExifTool::GPS::ToDMS($self, $val, 1, "N")',
     },
     'MetaDataList//Geolocation/Longitude' => {
         Name => 'GPSLongitude',
         Groups => { 2 => 'Location' },
-        PrintConv => q{
-            require Image::ExifTool::GPS;
-            Image::ExifTool::GPS::ToDMS($self, $val, 1, 'E');
-        },
+        PrintConv => 'Image::ExifTool::GPS::ToDMS($self, $val, 1, "E")',
     },
     'MetaDataList//Geolocation/MapDatum' => {
         Name => 'GPSMapDatum',
         Groups => { 2 => 'Location' },
     },
+    # slow motion stuff found in AAE files
+    'slowMotion/regions/timeRange/start/flags' => {
+        Name => 'SlowMotionRegionsStartTimeFlags',
+        PrintConv => { BITMASK => {
+            0 => 'Valid',
+            1 => 'Has been rounded',
+            2 => 'Positive infinity',
+            3 => 'Negative infinity',
+            4 => 'Indefinite',
+        }},
+    },
+    'slowMotion/regions/timeRange/start/value'     => 'SlowMotionRegionsStartTimeValue',
+    'slowMotion/regions/timeRange/start/timescale' => 'SlowMotionRegionsStartTimeScale',
+    'slowMotion/regions/timeRange/start/epoch'     => 'SlowMotionRegionsStartTimeEpoch',
+    'slowMotion/regions/timeRange/duration/flags'  => {
+        Name => 'SlowMotionRegionsDurationFlags',
+        PrintConv => { BITMASK => {
+            0 => 'Valid',
+            1 => 'Has been rounded',
+            2 => 'Positive infinity',
+            3 => 'Negative infinity',
+            4 => 'Indefinite',
+        }},
+    },
+    'slowMotion/regions/timeRange/duration/value'     => 'SlowMotionRegionsDurationValue',
+    'slowMotion/regions/timeRange/duration/timescale' => 'SlowMotionRegionsDurationTimeScale',
+    'slowMotion/regions/timeRange/duration/epoch'     => 'SlowMotionRegionsDurationEpoch',
+    'slowMotion/regions' => 'SlowMotionRegions',
+    'slowMotion/rate' => 'SlowMotionRate',
     XMLFileType => {
         # recognize MODD files by their content
         RawConv => q{
@@ -101,6 +130,11 @@ my %readProc = (
             return $val;
         },
     },
+    adjustmentData => { # AAE file
+        Name => 'AdjustmentData',
+        CompressedPLIST => 1,
+        SubDirectory => { TagTable => 'Image::ExifTool::PLIST::Main' },
+    },
 );
 
 #------------------------------------------------------------------------------
@@ -111,15 +145,15 @@ my %readProc = (
 # Returns: 1 if valid tag was found
 sub FoundTag($$$$;$)
 {
-    my ($exifTool, $tagTablePtr, $props, $val, $attrs) = @_;
+    my ($et, $tagTablePtr, $props, $val, $attrs) = @_;
     return 0 unless @$props;
-    my $verbose = $exifTool->Options('Verbose');
-    my $keys = $$exifTool{PListKeys} || ( $$exifTool{PListKeys} = [] );
+    my $verbose = $et->Options('Verbose');
+    my $keys = $$et{PListKeys} || ( $$et{PListKeys} = [] );
 
     my $prop = $$props[-1];
     if ($verbose > 1) {
-        $exifTool->VPrint(0, $$exifTool{INDENT}, '[', join('/',@$props), ' = ',
-            $exifTool->Printable($val), "]\n");
+        $et->VPrint(0, $$et{INDENT}, '[', join('/',@$props), ' = ',
+                    $et->Printable($val), "]\n");
     }
     # un-escape XML character entities
     $val = Image::ExifTool::XMP::UnescapeXML($val);
@@ -140,7 +174,7 @@ sub FoundTag($$$$;$)
         $val = ucfirst $prop;
     } else {
         # convert from UTF8 to ExifTool Charset
-        $val = $exifTool->Decode($val, 'UTF8');
+        $val = $et->Decode($val, 'UTF8');
         if ($prop eq 'key') {
             if (@$props <= 3) { # top-level key should be plist/dict/key
                 @$keys = ( $val );
@@ -159,7 +193,7 @@ sub FoundTag($$$$;$)
     my $tag = join '/', @$keys;     # generate tag ID from 'key' values
     my $tagInfo = $$tagTablePtr{$tag};
     unless ($tagInfo) {
-        $exifTool->VPrint(0, $exifTool->{INDENT}, "[adding $tag]\n") if $verbose;
+        $et->VPrint(0, $$et{INDENT}, "[adding $tag]\n") if $verbose;
         # generate tag name from ID
         my $name = $tag;
         $name =~ s{^MetaDataList//}{};  # shorten long MODD metadata tag names
@@ -174,12 +208,28 @@ sub FoundTag($$$$;$)
         AddTagToTable($tagTablePtr, $tag, $tagInfo);
     }
     # allow list-behaviour only for consecutive tags with the same ID
-    if ($$exifTool{LastPListTag} and $$exifTool{LastPListTag} ne $tagInfo) {
-        delete $$exifTool{LIST_TAGS}{$$exifTool{LastPListTag}};
+    if ($$et{LastPListTag} and $$et{LastPListTag} ne $tagInfo) {
+        delete $$et{LIST_TAGS}{$$et{LastPListTag}};
+    }
+    $$et{LastPListTag} = $tagInfo;
+    # override file type if applicable
+    $et->OverrideFileType($plistType{$tag}) if $plistType{$tag} and $$et{FILE_TYPE} eq 'XMP';
+    # handle compressed PLIST/JSON data
+    my $proc;
+    if ($$tagInfo{CompressedPLIST} and ref $val eq 'SCALAR' and $$val !~ /^bplist00/) {
+        if (eval { require IO::Uncompress::RawInflate }) {
+            my $inflated;
+            if (IO::Uncompress::RawInflate::rawinflate($val => \$inflated)) {
+                $val = \$inflated;
+            } else {
+                $et->Warn("Error inflating PLIST::$$tagInfo{Name}");
+            }
+        } else {
+            $et->Warn('Install IO::Uncompress to decode compressed PLIST data');
+        }
     }
-    $$exifTool{LastPListTag} = $tagInfo;
     # save the tag
-    $exifTool->HandleTag($tagTablePtr, $tag, $val);
+    $et->HandleTag($tagTablePtr, $tag, $val, ProcessProc => $proc);
 
     return 1;
 }
@@ -200,7 +250,7 @@ sub Get24u($$)
 # Returns: the object, or undef on error
 sub ExtractObject($$;$)
 {
-    my ($exifTool, $plistInfo, $parent) = @_;
+    my ($et, $plistInfo, $parent) = @_;
     my $raf = $$plistInfo{RAF};
     my ($buff, $val);
 
@@ -234,11 +284,11 @@ sub ExtractObject($$;$)
         # $size is the size of the remaining types
         if ($size == 0x0f) {
             # size is stored in extra integer object
-            $size = ExtractObject($exifTool, $plistInfo);
+            $size = ExtractObject($et, $plistInfo);
             return undef unless defined $size and $size =~ /^\d+$/;
         }
         if ($type == 4) {  # data
-            if ($size < 1000000 or $exifTool->Options('Binary')) {
+            if ($size < 1000000 or $et->Options('Binary')) {
                 $raf->Read($buff, $size) == $size or return undef;
             } else {
                 $buff = "Binary data $size bytes";
@@ -249,7 +299,7 @@ sub ExtractObject($$;$)
         } elsif ($type == 6) {  # UCS-2BE string
             $size *= 2;
             $raf->Read($buff, $size) == $size or return undef;
-            $val = $exifTool->Decode($buff, 'UCS2');
+            $val = $et->Decode($buff, 'UCS2');
         } elsif ($type == 10 or $type == 12 or $type == 13) { # array, set or dict
             # the remaining types store a list of references
             my $refSize = $$plistInfo{RefSize};
@@ -267,29 +317,42 @@ sub ExtractObject($$;$)
             if ($type == 13) { # dict
                 # prevent infinite recursion
                 if (defined $parent and length $parent > 1000) {
-                    $exifTool->WarnOnce('Possible deep recursion while parsing PLIST');
+                    $et->Warn('Possible deep recursion while parsing PLIST');
                     return undef;
                 }
                 my $tagTablePtr = $$plistInfo{TagTablePtr};
-                my $verbose = $exifTool->Options('Verbose');
+                my $verbose = $et->Options('Verbose');
+                $val = { }; # initialize return dictionary (will stay empty if tags are saved)
                 for ($i=0; $i<$size; ++$i) {
                     # get the entry key
                     $raf->Seek($$table[$refs[$i]], 0) or return undef;
-                    my $key = ExtractObject($exifTool, $plistInfo);
+                    my $key = ExtractObject($et, $plistInfo);
                     next unless defined $key and length $key; # silently ignore bad dict entries
                     # get the entry value
                     $raf->Seek($$table[$refs[$i+$size]], 0) or return undef;
                     # generate an ID for this tag
                     my $tag = defined $parent ? "$parent/$key" : $key;
                     undef $$plistInfo{DateFormat};
-                    my $val = ExtractObject($exifTool, $plistInfo, $tag);
-                    next if not defined $val or ref($val) eq 'HASH';
-                    my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
+                    my $obj = ExtractObject($et, $plistInfo, $tag);
+                    next if not defined $obj;
+                    unless ($tagTablePtr) {
+                        # make sure this is a valid structure field name
+                        if (not defined $key or $key !~ /^[-_a-zA-Z0-9]+$/) {
+                            $key = "Tag$i"; # (generate fake tag name if it had illegal characters)
+                        } elsif ($key !~ /^[_a-zA-Z]/) {
+                            $key = "_$key"; # (must begin with alpha or underline)
+                        }
+                        $$val{$key} = $obj if defined $obj;
+                        next;
+                    }
+                    next if ref($obj) eq 'HASH';
+                    my $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
                     unless ($tagInfo) {
-                        $exifTool->VPrint(0, $exifTool->{INDENT}, "[adding $tag]\n") if $verbose;
+                        $et->VPrint(0, $$et{INDENT}, "[adding $tag]\n") if $verbose;
                         my $name = $tag;
                         $name =~ s/([^A-Za-z])([a-z])/$1\u$2/g; # capitalize words
                         $name =~ tr/-_a-zA-Z0-9//dc; # remove illegal characters
+                        $name = 'Tag'.ucfirst($name) if length($name) < 2 or $name =~ /^[-0-9]/;
                         $tagInfo = { Name => ucfirst($name), List => 1 };
                         if ($$plistInfo{DateFormat}) {
                             $$tagInfo{Groups}{2} = 'Time';
@@ -298,18 +361,17 @@ sub ExtractObject($$;$)
                         AddTagToTable($tagTablePtr, $tag, $tagInfo);
                     }
                     # allow list-behaviour only for consecutive tags with the same ID
-                    if ($$exifTool{LastPListTag} and $$exifTool{LastPListTag} ne $tagInfo) {
-                        delete $$exifTool{LIST_TAGS}{$$exifTool{LastPListTag}};
+                    if ($$et{LastPListTag} and $$et{LastPListTag} ne $tagInfo) {
+                        delete $$et{LIST_TAGS}{$$et{LastPListTag}};
                     }
-                    $$exifTool{LastPListTag} = $tagInfo;
-                    $exifTool->HandleTag($tagTablePtr, $tag, $val);
+                    $$et{LastPListTag} = $tagInfo;
+                    $et->HandleTag($tagTablePtr, $tag, $obj);
                 }
-                $val = { }; # flag the value as a dictionary (ie. tags already saved)
             } else {
                 # extract the referenced objects
                 foreach $ref (@refs) {
                     $raf->Seek($$table[$ref], 0) or return undef;   # seek to this object
-                    $val = ExtractObject($exifTool, $plistInfo, $parent);
+                    $val = ExtractObject($et, $plistInfo, $parent);
                     next unless defined $val and ref $val ne 'HASH';
                     push @array, $val;
                 }
@@ -321,19 +383,30 @@ sub ExtractObject($$;$)
 }
 
 #------------------------------------------------------------------------------
-# Process binary PLIST file (ref 2)
+# Process binary PLIST data (ref 2)
 # Inputs: 0) ExifTool object ref, 1) DirInfo ref, 2) tag table ref
-# Returns: 1 on success
-sub ProcessBinaryPLIST($$$)
+# Returns: 1 on success (and returns plist value as $$dirInfo{Value})
+sub ProcessBinaryPLIST($$;$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    my $raf = $$dirInfo{RAF};
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my ($i, $buff, @table);
+    my $dataPt = $$dirInfo{DataPt};
 
-    $exifTool->VerboseDir('Binary PLIST');
+    $et->VerboseDir('Binary PLIST') unless $$dirInfo{NoVerboseDir};
     SetByteOrder('MM');
 
+    if ($dataPt) {
+        my $start = $$dirInfo{DirStart};
+        if ($start or ($$dirInfo{DirLen} and $$dirInfo{DirLen} != length $$dataPt)) {
+            my $buf2 = substr($$dataPt, $start || 0, $$dirInfo{DirLen});
+            $$dirInfo{RAF} = File::RandomAccess->new(\$buf2);
+        } else {
+            $$dirInfo{RAF} = File::RandomAccess->new($dataPt);
+        }
+        my $strt = $$dirInfo{DirStart} || 0;
+    }
     # read and parse the trailer
+    my $raf = $$dirInfo{RAF} or return 0;
     $raf->Seek(-32,2) and $raf->Read($buff,32)==32 or return 0;
     my $intSize = Get8u(\$buff, 6);
     my $refSize = Get8u(\$buff, 7);
@@ -360,44 +433,59 @@ sub ProcessBinaryPLIST($$$)
     );
     # position file pointer at the top object, and extract it
     $raf->Seek($table[$topObj], 0) or return 0;
-    my $result = ExtractObject($exifTool, \%plistInfo);
-    return defined $result ? 1 : 0;
+    $$dirInfo{Value} = ExtractObject($et, \%plistInfo);
+    return defined $$dirInfo{Value} ? 1 : 0;
 }
 
 #------------------------------------------------------------------------------
-# Extract information from a PLIST file
+# Extract information from a PLIST file (binary, XML or JSON format)
 # Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
 # Returns: 1 on success, 0 if this wasn't valid PLIST
 sub ProcessPLIST($$;$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my ($result, $notXML);
 
-    # process XML PLIST data using the XMP module
-    $$dirInfo{XMPParseOpts}{FoundProc} = \&FoundTag;
-    my $result = Image::ExifTool::XMP::ProcessXMP($exifTool, $dirInfo, $tagTablePtr);
-    delete $$dirInfo{XMPParseOpts};
-
-    unless ($result) {
-        my $buff;
-        my $raf = $$dirInfo{RAF} or return 0;
+    if ($dataPt) {
+        pos($$dataPt) = $$dirInfo{DirStart} || 0;
+        $notXML = 1 unless $$dataPt =~ /\G</g;
+    }
+    unless ($notXML) {
+        # process XML PLIST data using the XMP module
+        $$dirInfo{XMPParseOpts}{FoundProc} = \&FoundTag;
+        $result = Image::ExifTool::XMP::ProcessXMP($et, $dirInfo, $tagTablePtr);
+        delete $$dirInfo{XMPParseOpts};
+        return $result if $result;
+    }
+    my $buff;
+    my $raf = $$dirInfo{RAF};
+    if ($raf) {
         $raf->Seek(0,0) and $raf->Read($buff, 64) or return 0;
-        if ($buff =~ /^bplist0/) {
-            # binary PLIST file
-            my $tagTablePtr = GetTagTable('Image::ExifTool::PLIST::Main');
-            $exifTool->SetFileType('PLIST', 'application/x-plist');
-            $$exifTool{SET_GROUP1} = 'PLIST';
-            unless (ProcessBinaryPLIST($exifTool, $dirInfo, $tagTablePtr)) {
-                $exifTool->Error('Error reading binary PLIST file');
-            }
-            delete $$exifTool{SET_GROUP1};
-            $result = 1;
-        } elsif ($$exifTool{FILE_EXT} and $$exifTool{FILE_EXT} eq 'PLIST' and
-            $buff =~ /^\xfe\xff\x00/)
-        {
-            # (have seen very old PLIST files encoded as UCS-2BE with leading BOM)
-            $exifTool->Error('Old PLIST format currently not supported');
-            $result = 1;
+        $dataPt = \$buff;
+    } else {
+        return 0 unless $dataPt;
+    }
+    if ($$dataPt =~ /^bplist0/) {  # binary PLIST
+        # binary PLIST file
+        my $tagTablePtr = GetTagTable('Image::ExifTool::PLIST::Main');
+        $et->SetFileType('PLIST', 'application/x-plist');
+        $$et{SET_GROUP1} = 'PLIST';
+        unless (ProcessBinaryPLIST($et, $dirInfo, $tagTablePtr)) {
+            $et->Error('Error reading binary PLIST file');
         }
+        delete $$et{SET_GROUP1};
+        $result = 1;
+    } elsif ($$dataPt =~ /^\{"/) { # JSON PLIST
+        $raf and $raf->Seek(0);
+        require Image::ExifTool::JSON;
+        $result = Image::ExifTool::JSON::ProcessJSON($et, $dirInfo);
+    } elsif ($$et{FILE_EXT} and $$et{FILE_EXT} eq 'PLIST' and
+        $$dataPt =~ /^\xfe\xff\x00/)
+    {
+        # (have seen very old PLIST files encoded as UCS-2BE with leading BOM)
+        $et->Error('Old PLIST format currently not supported');
+        $result = 1;
     }
     return $result;
 }
@@ -425,7 +513,7 @@ This module decodes both the binary and XML-based PLIST format.
 
 =head1 AUTHOR
 
-Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2025, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
